@@ -190,6 +190,21 @@ if_hasconf(struct dhcpcd_ctx *ctx, const char *ifname)
 	return 0;
 }
 
+void
+if_markaddrsstale(struct if_head *ifs)
+{
+	struct interface *ifp;
+
+	TAILQ_FOREACH(ifp, ifs, next) {
+#ifdef INET
+		ipv4_markaddrsstale(ifp);
+#endif
+#ifdef INET6
+		ipv6_markaddrsstale(ifp, 0);
+#endif
+	}
+}
+
 void
 if_learnaddrs(struct dhcpcd_ctx *ctx, struct if_head *ifs,
     struct ifaddrs **ifaddrs)
@@ -234,7 +249,7 @@ if_learnaddrs(struct dhcpcd_ctx *ctx, struct if_head *ifs,
 #endif
 			ipv4_handleifa(ctx, RTM_NEWADDR, ifs, ifa->ifa_name,
 				&addr->sin_addr, &net->sin_addr,
-				brd ? &brd->sin_addr : NULL, addrflags);
+				brd ? &brd->sin_addr : NULL, addrflags, 0);
 			break;
 #endif
 #ifdef INET6
@@ -258,7 +273,7 @@ if_learnaddrs(struct dhcpcd_ctx *ctx, struct if_head *ifs,
 #endif
 			ipv6_handleifa(ctx, RTM_NEWADDR, ifs,
 			    ifa->ifa_name, &sin6->sin6_addr,
-			    ipv6_prefixlen(&net6->sin6_addr), addrflags);
+			    ipv6_prefixlen(&net6->sin6_addr), addrflags, 0);
 			break;
 #endif
 		}
@@ -268,6 +283,21 @@ if_learnaddrs(struct dhcpcd_ctx *ctx, struct if_head *ifs,
 	*ifaddrs = NULL;
 }
 
+void
+if_deletestaleaddrs(struct if_head *ifs)
+{
+	struct interface *ifp;
+
+	TAILQ_FOREACH(ifp, ifs, next) {
+#ifdef INET
+		ipv4_deletestaleaddrs(ifp);
+#endif
+#ifdef INET6
+		ipv6_deletestaleaddrs(ifp);
+#endif
+	}
+}
+
 bool
 if_valid_hwaddr(const uint8_t *hwaddr, size_t hwlen)
 {
